@@ -158,14 +158,13 @@ class ClusterMonitor(gevent.greenlet.Greenlet):
     another to listen to user requests.
     """
 
-    def __init__(self, fsid, cluster_name, notifier, persister, servers, eventer, requests):
+    def __init__(self, fsid, cluster_name, persister, servers, eventer, requests):
         super(ClusterMonitor, self).__init__()
 
         self.fsid = fsid
         self.name = cluster_name
         self.update_time = datetime.datetime.utcnow().replace(tzinfo=utc)
 
-        self._notifier = notifier
         self._persister = persister
         self._servers = servers
         self._eventer = eventer
@@ -388,7 +387,7 @@ class ClusterMonitor(gevent.greenlet.Greenlet):
 
     def _set_favorite(self, minion_id):
         assert minion_id != self._favorite_mon
-        self._requests.fail_all(minion_id)
+        self._requests.fail_all(self._favorite_mon, self.fsid)
 
         self._favorite_mon = minion_id
 
